import { black, white, gray, blue, green, orange, purple, red, yellow } from 'primer-colors';
import { fontSizes, lineHeights } from 'primer-typography';
import React from 'react';
import system from 'system-components/emotion';
import PropTypes from 'prop-types';
import classnames from 'classnames';
import Octicon, { TriangleDown, GitMerge, IssueClosed, IssueOpened, IssueReopened } from '@githubprimer/octicons-react';
import { style, themeGet } from 'styled-system';
import { arc, pie } from 'd3-shape';

var colors = {
  bodytext: gray[9],
  black: black,
  white: white,
  gray: gray,
  blue: blue,
  green: green,
  orange: orange,
  purple: purple,
  red: red,
  yellow: yellow,
  blackfade15: 'rgba(27, 31, 35, 0.15)',
  blackfade20: 'rgba(27, 31, 35, 0.20)',
  whitefade15: 'rgba(255, 255, 255, 0.15)',
  state: {
    error: red[5],
    failure: red[5],
    pending: yellow[7],
    queued: yellow[7],
    success: green[5],
    unknown: gray[4]
  }
};

var theme = {
  breakpoints: ['544px', '768px', '1012px', '1280px'],
  maxWidths: {
    small: '544px',
    medium: '768px',
    large: '1012px',
    xlarge: '1280px'
  },
  fonts: {
    normal: fontStack(['-apple-system', 'BlinkMacSystemFont', 'Segoe UI', 'Helvetica', 'Arial', 'sans-serif', 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol']),
    mono: fontStack(['SFMono-Regular', 'Consolas', 'Liberation Mono', 'Menlo', 'Courier', 'monospace'])
  },
  colors: colors,
  borders: [0, '1px solid'],
  fontSizes: fontSizes,
  lineHeights: lineHeights,
  radii: [0, 3, 6],
  shadows: {
    small: '0 1px 1px rgba(27, 31, 35, 0.1)',
    medium: '0 1px 5px rgba(27, 31, 35, 0.15)',
    large: '0 1px 15px rgba(27, 31, 35, 0.15)',
    'extra-large': '0 10px 50px rgba(27, 31, 35, 0.07)'
  },
  space: [0, 4, 8, 16, 24, 32, 40, 48]
};

function fontStack(fonts) {
  return fonts.map(function (font) {
    return font.includes(' ') ? '"' + font + '"' : font;
  }).join(', ');
}

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) {
  return typeof obj;
} : function (obj) {
  return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;
};

var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();

var defineProperty = function (obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
};

var _extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};

var inherits = function (subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
  }

  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      enumerable: false,
      writable: true,
      configurable: true
    }
  });
  if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
};

var objectWithoutProperties = function (obj, keys) {
  var target = {};

  for (var i in obj) {
    if (keys.indexOf(i) >= 0) continue;
    if (!Object.prototype.hasOwnProperty.call(obj, i)) continue;
    target[i] = obj[i];
  }

  return target;
};

var possibleConstructorReturn = function (self, call) {
  if (!self) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return call && (typeof call === "object" || typeof call === "function") ? call : self;
};

var slicedToArray = function () {
  function sliceIterator(arr, i) {
    var _arr = [];
    var _n = true;
    var _d = false;
    var _e = undefined;

    try {
      for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
        _arr.push(_s.value);

        if (i && _arr.length === i) break;
      }
    } catch (err) {
      _d = true;
      _e = err;
    } finally {
      try {
        if (!_n && _i["return"]) _i["return"]();
      } finally {
        if (_d) throw _e;
      }
    }

    return _arr;
  }

  return function (arr, i) {
    if (Array.isArray(arr)) {
      return arr;
    } else if (Symbol.iterator in Object(arr)) {
      return sliceIterator(arr, i);
    } else {
      throw new TypeError("Invalid attempt to destructure non-iterable instance");
    }
  };
}();

var toConsumableArray = function (arr) {
  if (Array.isArray(arr)) {
    for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) arr2[i] = arr[i];

    return arr2;
  } else {
    return Array.from(arr);
  }
};

var COMMON = ['color', 'space'];

var TYPOGRAPHY = COMMON.concat(
// typography props
'fontFamily', 'fontSize', 'fontWeight', 'lineHeight');

var LAYOUT = COMMON.concat(
// layout props
'borders', 'borderColor', 'borderRadius', 'boxShadow', 'display', 'size', 'width', 'height', 'minWidth', 'minHeight', 'maxWidth', 'maxHeight', 'verticalAlign');

var FLEX_CONTAINER = LAYOUT.concat(
// flex container props (display: flex)
'alignContent', 'alignItems', 'flexWrap', 'flex', 'flexBasis', 'flexDirection', 'justifyContent', 'order');

var FLEX_ITEM = LAYOUT.concat(
// flex container child props
'justifySelf', 'alignSelf');

/**
 * Defensively determine whether a component function or class is a "system
 * component" by checking its `systemComponent` flag or whether its
 * `defaultProps.blacklist` is an array.
 */
function isSystemComponent(Component) {
  return Component.systemComponent === true || Component.defaultProps && Array.isArray(Component.defaultProps.blacklist);
}

/**
 * Create a "system component" with the named props from styled-system.
 * The Component (first) argument can either be a React component (a function
 * or a class) or an object representing default props. To pass a custom
 * component with other default props, set the object's `is` key to the
 * component:
 *
 * ```js
 * const Wrapped = withSystemProps({is: Component, m: 2})
 * ```
 *
 * which is the equivalent of:
 *
 * ```js
 * const Wrapped = withSystemProps(Component)
 * Wrapped.defaultProps = {
 *   m: 2
 * }
 * ```
 */
function withSystemProps(Component) {
  var props = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : COMMON;

  if (isSystemComponent(Component)) {
    throw new Error(Component.name + ' is already a system component; can\'t call withSystemProps() on it');
  }

  var component = (typeof Component === 'undefined' ? 'undefined' : _typeof(Component)) === 'object' ? Component : { is: Component };
  if (typeof component.is === 'function') {
    component.is = guardDoubleRender(component.is);
  }

  var Wrapped = system.apply(undefined, [component].concat(toConsumableArray(props)));
  Wrapped.displayName = Component.displayName;
  Object.assign(Wrapped.propTypes, Component.propTypes);

  // Copy over non-system keys from components
  // eg. Tooltip.js => Tooltip.directions Tooltip.alignments
  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = Object.keys(Component)[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var key = _step.value;

      if (!Wrapped.hasOwnProperty(key)) {
        Wrapped[key] = Component[key];
      }
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }

  return withDefaultTheme(Wrapped);
}

/**
 * Set the component's defaultProps.theme to our theme, and returns the
 * component.
 */
function withDefaultTheme(Component) {
  var theme$$1 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : theme;

  if (Component.defaultProps) {
    Component.defaultProps.theme = theme$$1;
  } else {
    Component.defaultProps = { theme: theme$$1 };
  }
  return Component;
}

/**
 * Remove the named keys from a component's propTypes object (if present), and
 * return the component.
 */
function withoutPropTypes(Component, props) {
  var _iteratorNormalCompletion2 = true;
  var _didIteratorError2 = false;
  var _iteratorError2 = undefined;

  try {
    for (var _iterator2 = props[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
      var prop = _step2.value;

      delete Component.propTypes[prop];
    }
  } catch (err) {
    _didIteratorError2 = true;
    _iteratorError2 = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion2 && _iterator2.return) {
        _iterator2.return();
      }
    } finally {
      if (_didIteratorError2) {
        throw _iteratorError2;
      }
    }
  }

  return Component;
}

function guardDoubleRender(Component) {
  function render(props) {
    var is = props.is,
        rest = objectWithoutProperties(props, ['is']);

    if (is === Component || is === render) {
      return React.createElement(Component, rest);
    } else {
      return React.createElement(Component, props);
    }
  }
  return render;
}

var Block = withSystemProps('div', LAYOUT);

var Box = withSystemProps({
  is: 'div',
  bg: 'white',
  border: 1,
  borderColor: 'gray.2',
  borderRadius: 1
}, LAYOUT);

function Avatar(_ref) {
  var alt = _ref.alt,
      isChild = _ref.isChild,
      _ref$size = _ref.size,
      size = _ref$size === undefined ? 20 : _ref$size,
      src = _ref.src,
      className = _ref.className;

  var classes = classnames('avatar', {
    'avatar-small': size <= 24,
    'avatar-child': isChild
  }, className);

  return React.createElement('img', { className: classes, alt: alt, src: src, width: size, height: size });
}

Avatar.propTypes = {
  alt: PropTypes.string,
  isChild: PropTypes.bool,
  size: PropTypes.number,
  src: PropTypes.string
};

var Avatar$1 = withSystemProps(Avatar, COMMON);

function Button(_ref) {
  var Tag = _ref.is,
      children = _ref.children,
      size = _ref.size,
      block = _ref.block,
      linkStyle = _ref.linkStyle,
      grouped = _ref.grouped,
      scheme = _ref.scheme,
      onClick = _ref.onClick,
      disabled = _ref.disabled,
      className = _ref.className,
      rest = objectWithoutProperties(_ref, ['is', 'children', 'size', 'block', 'linkStyle', 'grouped', 'scheme', 'onClick', 'disabled', 'className']);

  var classes = classnames(className, {
    btn: !linkStyle,
    'btn-link': linkStyle,
    'btn-sm': size === 'sm',
    'btn-large': size === 'large',
    'btn-block': block,
    'BtnGroup-item': grouped
  }, scheme ? 'btn-' + scheme : null);

  return React.createElement(
    Tag,
    _extends({}, rest, { type: 'button', disabled: disabled, onClick: disabled ? undefined : onClick, className: classes }),
    children
  );
}

Button.defaultProps = {
  is: 'button'
};

Button.propTypes = {
  block: PropTypes.bool,
  children: PropTypes.node,
  disabled: PropTypes.bool,
  grouped: PropTypes.bool,
  is: PropTypes.oneOfType([PropTypes.oneOf(['button', 'a', 'summary', 'input']), PropTypes.func]),
  linkStyle: PropTypes.bool,
  onClick: PropTypes.func,
  scheme: PropTypes.string,
  size: PropTypes.oneOf(['sm', 'large'])
};

var Button$1 = withSystemProps(Button, COMMON);

var ButtonDanger = function ButtonDanger(props) {
  return React.createElement(Button$1, _extends({}, props, { scheme: 'danger' }));
};

var ButtonPrimary = function ButtonPrimary(props) {
  return React.createElement(Button$1, _extends({}, props, { scheme: 'primary' }));
};

var ButtonOutline = function ButtonOutline(props) {
  return React.createElement(Button$1, _extends({}, props, { scheme: 'outline' }));
};

var ButtonLink = function ButtonLink(props) {
  return React.createElement(Button$1, _extends({}, props, { linkStyle: true }));
};

function OcticonButton(_ref) {
  var disabled = _ref.disabled,
      className = _ref.className,
      icon = _ref.icon,
      label = _ref.label,
      onClick = _ref.onClick,
      size = _ref.size;

  var buttonProps = {
    'aria-label': label,
    className: classnames('btn-link text-inherit', className),
    disabled: disabled,
    onClick: onClick
  };
  var octiconProps = { icon: icon, size: size };
  return React.createElement(
    'button',
    buttonProps,
    React.createElement(Octicon, octiconProps)
  );
}

OcticonButton.defaultProps = {
  label: ''
};

OcticonButton.propTypes = {
  disabled: PropTypes.bool,
  icon: Octicon.propTypes.icon,
  label: PropTypes.string.isRequired,
  onClick: PropTypes.func,
  size: Octicon.propTypes.size
};

var OcticonButton$1 = withSystemProps(OcticonButton, COMMON);

var oppositeEdge = {
  top: 'Bottom',
  right: 'Left',
  bottom: 'Top',
  left: 'Right'
};

var perpendicularEdge = {
  top: 'Left',
  right: 'Top',
  bottom: 'Left',
  left: 'Top'
};

function getEdgeAlign(location) {
  var _location$split = location.split('-'),
      _location$split2 = slicedToArray(_location$split, 2),
      edge = _location$split2[0],
      align = _location$split2[1];

  return [edge, align];
}

function getPosition(edge, align, spacing) {
  var _ref;

  var opposite = oppositeEdge[edge].toLowerCase();
  var perp = perpendicularEdge[edge].toLowerCase();
  return _ref = {}, defineProperty(_ref, opposite, '100%'), defineProperty(_ref, align || perp, align ? spacing : '50%'), _ref;
}

var getBg = style({ prop: 'bg', key: 'colors' });
var getBorderColor = style({ prop: 'borderColor', key: 'colors' });
var getBorderWidth = style({ prop: 'borderWidth', key: 'borderWidths', scale: [0, 1] });
var getSize = style({ prop: 'size', key: 'space' });

function Caret(props) {
  var _getBg = getBg(props),
      bg = _getBg.bg;

  var _getBorderColor = getBorderColor(props),
      borderColor = _getBorderColor.borderColor;

  var _getBorderWidth = getBorderWidth(props),
      borderWidth = _getBorderWidth.borderWidth;

  var _getSize = getSize(props),
      size = _getSize.size;

  var location = props.location;

  var _getEdgeAlign = getEdgeAlign(location),
      _getEdgeAlign2 = slicedToArray(_getEdgeAlign, 2),
      edge = _getEdgeAlign2[0],
      align = _getEdgeAlign2[1];

  var perp = perpendicularEdge[edge];

  var style$$1 = _extends({
    pointerEvents: 'none',
    position: 'absolute'
  }, getPosition(edge, align, size), defineProperty({}, 'margin' + perp, align ? null : -size));

  // note: these arrays represent points in the form [x, y]
  var a = [-size, 0];
  var b = [0, size];
  var c = [size, 0];

  // spaces are optional in path `d` attribute, and points are
  // represented in the form `x,y` -- which is what the arrays above
  // become when stringified!
  var triangle = 'M' + a + 'L' + b + 'L' + c + 'L' + a + 'Z';
  var line = 'M' + a + 'L' + b + 'L' + c;

  var transform = {
    top: 'translate(' + [size, size * 2] + ') rotate(180)',
    right: 'translate(' + [0, size] + ') rotate(-90)',
    bottom: 'translate(' + [size, 0] + ')',
    left: 'translate(' + [size * 2, size] + ') rotate(90)'
  }[edge];

  return React.createElement(
    'svg',
    { width: size * 2, height: size * 2, style: style$$1 },
    React.createElement(
      'g',
      { transform: transform },
      React.createElement('path', { d: triangle, fill: bg }),
      React.createElement('path', { d: line, fill: 'none', stroke: borderColor, strokeWidth: borderWidth })
    )
  );
}

Caret.locations = ['top', 'top-left', 'top-right', 'right', 'right-top', 'right-bottom', 'bottom', 'bottom-left', 'bottom-right', 'left', 'left-top', 'left-bottom'];

Caret.defaultProps = {
  bg: Box.defaultProps.bg,
  borderColor: Box.defaultProps.borderColor,
  borderWidth: 1,
  location: 'bottom',
  size: 2
};

Caret.propTypes = {
  /* eslint-disable react/sort-prop-types  */
  // eslint can't determine whether these props are used
  // because they're accessed inside of styled-system.
  /* eslint-disable react/no-unused-prop-types */
  bg: PropTypes.string,
  borderColor: PropTypes.string,
  borderWidth: PropTypes.number,
  size: PropTypes.number,
  location: PropTypes.oneOf(Caret.locations)
  /* eslint-enable */
};

var Caret$1 = withDefaultTheme(Caret);

function CaretBox(_ref) {
  var position = _ref.position,
      rest = objectWithoutProperties(_ref, ['position']);

  // don't destructure these, just grab them
  var bg = rest.bg,
      border = rest.border,
      borderColor = rest.borderColor;
  var caret = rest.caret,
      children = rest.children,
      boxProps = objectWithoutProperties(rest, ['caret', 'children']);

  var caretProps = { bg: bg, borderColor: borderColor, borderWidth: border, location: caret };
  return React.createElement(
    Box,
    _extends({}, boxProps, { css: { position: position } }),
    children,
    React.createElement(Caret$1, caretProps)
  );
}

CaretBox.propTypes = _extends({}, Box.propTypes, {
  caret: Caret$1.propTypes.location
});

CaretBox.defaultProps = _extends({}, Box.defaultProps, {
  position: 'relative'

  // we can set this because it "extends" Box implicitly
});CaretBox.systemComponent = true;

var FlexContainer = withSystemProps({
  is: 'div',
  display: 'flex'
}, FLEX_CONTAINER);

function CircleOcticon(props) {
  var size = props.size;
  var icon = props.icon,
      rest = objectWithoutProperties(props, ['icon']);

  return React.createElement(
    FlexContainer,
    _extends({}, rest, { size: size, alignItems: 'center', justifyContent: 'center' }),
    React.createElement(Octicon, { icon: icon, size: size })
  );
}

CircleOcticon.defaultProps = _extends({}, FlexContainer.defaultProps, {
  size: 32,
  borderRadius: '50%'
});

CircleOcticon.propTypes = _extends({}, FlexContainer.propTypes, {
  icon: Octicon.propTypes.icon,
  size: PropTypes.number
});

var ICON_CLASS = 'CircleBadge-icon';

var CircleBadge = function CircleBadge(_ref) {
  var _ref$is = _ref.is,
      Tag = _ref$is === undefined ? 'div' : _ref$is,
      _ref$size = _ref.size,
      size = _ref$size === undefined ? 'medium' : _ref$size,
      bg = _ref.bg,
      children = _ref.children,
      className = _ref.className,
      rest = objectWithoutProperties(_ref, ['is', 'size', 'bg', 'children', 'className']);

  var mappedChildren = React.Children.map(children, function (child) {
    var _child$props$classNam = child.props.className,
        className = _child$props$classNam === undefined ? '' : _child$props$classNam;

    if (!className.includes(ICON_CLASS)) {
      className = classnames(ICON_CLASS, className);
    }
    return React.cloneElement(child, { className: className });
  });
  var classes = classnames(className, 'CircleBadge', 'CircleBadge--' + size, bg && 'bg-' + bg);
  return React.createElement(
    Tag,
    _extends({ className: classes }, rest),
    mappedChildren
  );
};

CircleBadge.propTypes = {
  alt: PropTypes.string,
  bg: PropTypes.string,
  is: PropTypes.oneOfType([PropTypes.string, PropTypes.func]),
  size: PropTypes.oneOf(['small', 'medium', 'large']),
  src: PropTypes.string
};

var CircleBadge$1 = withSystemProps(CircleBadge, COMMON);

var Details = function (_React$Component) {
  inherits(Details, _React$Component);

  function Details(props) {
    classCallCheck(this, Details);

    var _this = possibleConstructorReturn(this, (Details.__proto__ || Object.getPrototypeOf(Details)).call(this, props));

    _this.state = { open: Boolean(props.open) };
    _this.toggle = _this.toggle.bind(_this);
    return _this;
  }

  createClass(Details, [{
    key: 'toggle',
    value: function toggle(event) {
      if (event) {
        event.preventDefault();
      }
      this.setState({ open: !this.state.open });
    }
  }, {
    key: 'render',
    value: function render() {
      var _props = this.props,
          className = _props.className,
          children = _props.children,
          _props$render = _props.render,
          render = _props$render === undefined ? getRenderer(children) : _props$render,
          rest = objectWithoutProperties(_props, ['className', 'children', 'render']);
      var open = this.state.open;


      return React.createElement(
        'details',
        _extends({}, rest, { className: classnames('details-reset', className), open: open }),
        render({ open: open, toggle: this.toggle })
      );
    }
  }]);
  return Details;
}(React.Component);

function getRenderer(children) {
  return typeof children === 'function' ? children : function () {
    return children;
  };
}

Details.propTypes = {
  children: PropTypes.oneOfType([PropTypes.node, PropTypes.func]),
  className: PropTypes.string,
  open: PropTypes.bool,
  render: PropTypes.func
};

var Details$1 = withoutPropTypes(withSystemProps(Details, COMMON), ['is']);

var Octicon$1 = withSystemProps(Octicon, COMMON);

function Dropdown(_ref) {
  var title = _ref.title,
      scheme = _ref.scheme,
      children = _ref.children,
      className = _ref.className,
      rest = objectWithoutProperties(_ref, ['title', 'scheme', 'children', 'className']);
  var minWidth = rest.minWidth;

  return React.createElement(
    'div',
    _extends({ className: classnames(className, 'BtnGroup') }, rest),
    React.createElement(
      FlexContainer,
      { is: Details$1, className: 'BtnGroup-form', css: { position: 'relative' } },
      function (_ref2) {
        var toggle = _ref2.toggle;
        return React.createElement(
          React.Fragment,
          null,
          React.createElement(
            Button$1,
            { is: 'summary', scheme: scheme, grouped: true, onClick: toggle },
            title,
            React.createElement(Octicon$1, { icon: TriangleDown, size: 14, ml: title ? 2 : 0 })
          ),
          React.createElement(
            Block,
            {
              bg: 'white',
              border: 1,
              borderColor: 'gray.2',
              borderRadius: 1,
              boxShadow: 'small',
              mt: 1,
              px: 3,
              py: 2,
              minWidth: minWidth,
              css: { position: 'absolute', zIndex: 99999 }
            },
            children,
            React.createElement(Caret$1, { location: 'top-left' })
          )
        );
      }
    )
  );
}

Dropdown.propTypes = {
  children: PropTypes.node,
  scheme: Button$1.propTypes.scheme,
  title: PropTypes.string
};

var Dropdown$1 = withSystemProps(Dropdown, COMMON);

var defaultColor = '#666';
var getStateColors = themeGet('colors.state', {});

function DonutSlice(props) {
  var children = props.children,
      d = props.d,
      fill = props.fill,
      state = props.state,
      value = props.value;

  var stateColors = getStateColors(props);
  var color = fill || stateColors[state] || stateColors.unknown || defaultColor;
  return React.createElement(
    'path',
    { d: d, fill: color, 'data-value': value },
    children
  );
}

DonutSlice.propTypes = {
  // <title> is really the only thing that should be acceptable here
  children: PropTypes.shape({ type: 'title' }),
  d: PropTypes.string,
  fill: PropTypes.string,
  state: PropTypes.string,
  /* eslint-disable react/no-unused-prop-types */
  theme: PropTypes.shape({
    colors: PropTypes.shape({
      state: PropTypes.objectOf(PropTypes.string)
    })
  }),
  /* eslint-enable */
  value: PropTypes.number
};

var DonutSlice$1 = withDefaultTheme(DonutSlice);

function DonutChart(props) {
  var className = props.className,
      data = props.data,
      _props$children = props.children,
      children = _props$children === undefined ? mapData(data) : _props$children,
      size = props.size;


  var radius = size / 2;
  var innerRadius = radius - 6;

  var pie$$1 = pie().value(function (child) {
    return child.props.value;
  });

  // coerce the children into an array
  var childList = React.Children.toArray(children);
  var arcData = pie$$1(childList);
  var arc$$1 = arc().innerRadius(innerRadius).outerRadius(radius);

  var slices = childList.map(function (child, i) {
    return React.cloneElement(child, { d: arc$$1(arcData[i]) });
  });

  return React.createElement(
    'svg',
    { width: size, height: size, className: className },
    React.createElement(
      'g',
      { transform: 'translate(' + radius + ',' + radius + ')' },
      slices
    )
  );
}

function mapData(data) {
  return Object.keys(data).map(function (key) {
    return React.createElement(DonutSlice$1, { key: key, state: key, value: data[key] });
  });
}

DonutChart.defaultProps = {
  size: 30
};

DonutChart.propTypes = {
  // require elements, not mixed content: <DonutSlice>, <title>, etc.
  children: PropTypes.oneOfType([PropTypes.element, PropTypes.arrayOf(PropTypes.element)]),
  data: PropTypes.objectOf(PropTypes.number),
  size: PropTypes.number
};

var DonutChart$1 = withSystemProps(DonutChart, ['space']);

var ITEM_CLASS = 'filter-item';
var SELECTED_CLASS = 'selected';

function FilterList(_ref) {
  var children = _ref.children,
      className = _ref.className,
      small = _ref.small;

  var classes = classnames(className, 'filter-list', small && 'small');

  var items = React.Children.map(children, function (child) {
    return React.createElement(
      'li',
      null,
      child
    );
  });

  return React.createElement(
    'ul',
    { className: classes },
    items
  );
}

Object.assign(FilterList, { ITEM_CLASS: ITEM_CLASS, SELECTED_CLASS: SELECTED_CLASS });

FilterList.defaultProps = {
  m: 0,
  p: 0
};

FilterList.propTypes = {
  children: PropTypes.node,
  small: PropTypes.bool
};

var FilterList$1 = withSystemProps(FilterList, COMMON);

function getCountComponent(count) {
  return React.createElement(
    'span',
    { className: 'count', title: 'results' },
    count
  );
}

function FilterListItem(_ref) {
  var children = _ref.children,
      className = _ref.className,
      count = _ref.count,
      selected = _ref.selected,
      Tag = _ref.is,
      rest = objectWithoutProperties(_ref, ['children', 'className', 'count', 'selected', 'is']);

  var classes = classnames(ITEM_CLASS, selected && SELECTED_CLASS, className);

  if (typeof rest.to === 'string') {
    rest.activeClassName = SELECTED_CLASS;
  }

  return React.createElement(
    Tag,
    _extends({ className: classes }, rest),
    count && getCountComponent(count),
    children
  );
}

FilterListItem.defaultProps = {
  is: 'a'
};

FilterListItem.propTypes = {
  children: PropTypes.node,
  className: PropTypes.string,
  count: PropTypes.string,
  is: PropTypes.oneOfType([PropTypes.string, PropTypes.func]),
  selected: PropTypes.bool
};

var FilterListItem$1 = withSystemProps(FilterListItem, COMMON);

var FlexItem = withSystemProps('div', FLEX_ITEM);

function TextInput(_ref) {
  var autocomplete = _ref.autocomplete,
      block = _ref.block,
      className = _ref.className,
      disabled = _ref.disabled,
      id = _ref.id,
      name = _ref.name,
      onChange = _ref.onChange,
      placeholder = _ref.placeholder,
      required = _ref.required,
      size = _ref.size,
      value = _ref.value;

  var classes = classnames(className, 'form-control', {
    'input-block': block,
    'input-sm': size === 'small',
    'input-lg': size === 'large'
  });
  var inputProps = {
    className: classes,
    'aria-label': placeholder,
    autoComplete: autocomplete,
    onChange: onChange,
    disabled: disabled,
    id: id,
    name: name,
    placeholder: placeholder,
    required: required,
    value: value,
    type: 'text'
  };
  return React.createElement('input', inputProps);
}

TextInput.propTypes = {
  autocomplete: PropTypes.string,
  block: PropTypes.bool,
  disabled: PropTypes.bool,
  id: PropTypes.string,
  name: PropTypes.string,
  onChange: PropTypes.func,
  placeholder: PropTypes.string,
  required: PropTypes.bool,
  size: PropTypes.oneOf(['small', 'large']),
  value: PropTypes.string
};

var TextInput$1 = withSystemProps(TextInput, COMMON);

var Heading = withSystemProps({
  is: 'h1',
  fontSize: 5,
  m: 0
}, TYPOGRAPHY);

var colorScheme = function colorScheme(scheme, outline) {
  if (outline) {
    return {
      'Label--outline-green': scheme === 'green'
    };
  } else {
    return {
      'Label--gray': scheme == null || scheme === 'gray',
      'Label--gray-darker': scheme === 'gray-darker',
      'Label--orange': scheme === 'orange',
      'bg-green': scheme === 'green'
    };
  }
};

function Label(_ref) {
  var className = _ref.className,
      outline = _ref.outline,
      scheme = _ref.scheme,
      rest = objectWithoutProperties(_ref, ['className', 'outline', 'scheme']);

  var classes = classnames(className, 'Label', outline && 'Label--outline', colorScheme(scheme, outline));
  return React.createElement('span', _extends({ className: classes }, rest));
}

Label.propTypes = {
  outline: PropTypes.bool,
  scheme: PropTypes.oneOf(['gray', 'gray-darker', 'green', 'orange'])
};

var Label$1 = withSystemProps(Label, ['space']);

function BranchName(_ref) {
  var children = _ref.children,
      href = _ref.href,
      Tag = _ref.is,
      className = _ref.className;

  // We don't want someone to use href on a non tag
  if (Tag !== 'a') {
    href = null;
  }

  return React.createElement(
    Tag,
    { href: href, className: classnames('branch-name', className) },
    children
  );
}

BranchName.defaultProps = {
  is: 'a'
};

BranchName.propTypes = {
  children: PropTypes.node,
  href: PropTypes.string,
  is: PropTypes.oneOfType([PropTypes.string, PropTypes.func])
};

var BranchName$1 = withSystemProps(BranchName, COMMON);

function Link(_ref) {
  var children = _ref.children,
      className = _ref.className,
      muted = _ref.muted,
      scheme = _ref.scheme,
      nounderline = _ref.nounderline,
      rest = objectWithoutProperties(_ref, ['children', 'className', 'muted', 'scheme', 'nounderline']);

  var colorClass = scheme ? 'link-' + scheme : muted ? 'muted-link' : 'text-blue';
  var classes = classnames(className, colorClass, nounderline && 'no-underline');
  return React.createElement(
    'a',
    _extends({ className: classes }, rest),
    children
  );
}

Link.propTypes = {
  href: PropTypes.string,
  muted: PropTypes.bool,
  nounderline: PropTypes.bool,
  scheme: PropTypes.oneOf(['gray', 'gray-dark'])
};

var Link$1 = withSystemProps(Link, COMMON);

var stateColorMap = {
  open: 'green',
  opened: 'green',
  reopened: 'green',
  closed: 'red',
  merged: 'purple'
};

var stateOcticonMap = {
  open: IssueOpened,
  opened: IssueOpened,
  reopened: IssueReopened,
  closed: IssueClosed,
  merged: GitMerge
};

function getOcticon(state) {
  if (!state) {
    return null;
  }
  return React.createElement(Octicon, { icon: stateOcticonMap[state] });
}

function getIconComponent(icon, children) {
  if (icon && children) {
    return React.createElement(
      'span',
      { className: 'mr-1' },
      icon
    );
  } else if (icon) {
    return React.createElement(
      'span',
      { className: 'd-flex m-1' },
      icon
    );
  }
  return null;
}

function StateLabel(_ref) {
  var state = _ref.state,
      className = _ref.className,
      scheme = _ref.scheme,
      icon = _ref.icon,
      small = _ref.small,
      children = _ref.children;

  if (icon !== false) {
    icon = icon || getOcticon(state);
  }

  var color = scheme || stateColorMap[state];
  var styleProps = {};
  if (color === 'yellow') {
    styleProps.style = { backgroundColor: colors.yellow[7] };
  }
  var iconComponent = getIconComponent(icon, children);
  var classes = classnames(className, 'State', {
    'State--small': small
  }, color && color !== 'yellow' ? 'State--' + color : null);

  return React.createElement(
    'span',
    _extends({ className: classes }, styleProps),
    iconComponent,
    children
  );
}

StateLabel.propTypes = {
  children: PropTypes.node,
  icon: PropTypes.oneOfType([PropTypes.node, PropTypes.bool]),
  scheme: PropTypes.string,
  small: PropTypes.bool,
  state: PropTypes.oneOf(Object.keys(stateOcticonMap))
};

var StateLabel$1 = withSystemProps(StateLabel, COMMON);

var stateColorMap$1 = {
  ready: 'green',
  invalid: 'invalid',
  merged: 'purple',
  pending: 'yellow'
};

var octicon = React.createElement(Octicon, { icon: GitMerge, size: 'medium' });

function MergeStatus(_ref) {
  var state = _ref.state,
      rest = objectWithoutProperties(_ref, ['state']);

  return React.createElement(StateLabel$1, _extends({}, rest, { scheme: stateColorMap$1[state], icon: octicon }));
}

MergeStatus.propTypes = {
  state: PropTypes.oneOf(['ready', 'invalid', 'merged', 'pending']).isRequired
};

var MergeStatus$1 = withSystemProps(MergeStatus, COMMON);

var Text = withSystemProps('span', TYPOGRAPHY);

function Tooltip(_ref) {
  var direction = _ref.direction,
      children = _ref.children,
      className = _ref.className,
      text = _ref.text,
      noDelay = _ref.noDelay,
      align = _ref.align,
      wrap = _ref.wrap;

  var classes = classnames(className, 'tooltipped', 'tooltipped-' + direction, align && 'tooltipped-align-' + align + '-2', noDelay && 'tooltipped-no-delay', wrap && 'tooltipped-multiline');
  return React.createElement(
    'span',
    { 'aria-label': text, className: classes },
    children
  );
}

Tooltip.alignments = ['left', 'right'];

Tooltip.directions = ['n', 'ne', 'e', 'se', 's', 'sw', 'w', 'nw'];

Tooltip.defaultProps = {
  direction: 'n'
};

Tooltip.propTypes = {
  align: PropTypes.oneOf(Tooltip.alignments),
  children: PropTypes.node,
  direction: PropTypes.oneOf(Tooltip.directions),
  noDelay: PropTypes.bool,
  text: PropTypes.string,
  wrap: PropTypes.bool
};

var Tooltip$1 = withSystemProps(Tooltip, COMMON);

function CounterLabel(_ref) {
  var scheme = _ref.scheme,
      children = _ref.children,
      className = _ref.className;

  return React.createElement(
    'span',
    { className: classnames(className, 'Counter', scheme && 'Counter--' + scheme) },
    children
  );
}

CounterLabel.propTypes = {
  children: PropTypes.node,
  scheme: PropTypes.oneOf(['gray', 'gray-light'])
};

var CounterLabel$1 = withSystemProps(CounterLabel, COMMON);

var schemeMap = {
  green: 'success',
  red: 'error',
  yellow: 'warn'
};

function Flash(_ref) {
  var children = _ref.children,
      className = _ref.className,
      full = _ref.full,
      scheme = _ref.scheme;

  var classes = classnames(className, 'flash', full && 'flash-full', scheme && 'flash-' + schemeMap[scheme]);
  return React.createElement(
    'div',
    { className: classes },
    children
  );
}

Flash.propTypes = {
  children: PropTypes.node,
  full: PropTypes.bool,
  scheme: PropTypes.oneOf(Object.keys(schemeMap))
};

var Flash$1 = withSystemProps(Flash, COMMON);

var ITEM_CLASS$1 = 'UnderlineNav-item no-underline';
var SELECTED_CLASS$1 = 'selected';

function UnderlineNav(_ref) {
  var actions = _ref.actions,
      className = _ref.className,
      align = _ref.align,
      children = _ref.children,
      full = _ref.full,
      label = _ref.label;

  var classes = classnames(className, 'UnderlineNav', align && 'UnderlineNav--' + align, full && 'UnderlineNav--full');
  return React.createElement(
    'nav',
    { className: classes, 'aria-label': label },
    React.createElement(
      'div',
      { className: 'UnderlineNav-body' },
      children
    ),
    actions && React.createElement(
      'div',
      { className: 'UnderlineNav-actions' },
      actions
    )
  );
}

// make it possible to destructure these from UnderlineNav:
// const {ITEM_CLASS} = UnderlineNav
Object.assign(UnderlineNav, { ITEM_CLASS: ITEM_CLASS$1, SELECTED_CLASS: SELECTED_CLASS$1 });

UnderlineNav.propTypes = {
  actions: PropTypes.node,
  align: PropTypes.oneOf(['right']),
  children: PropTypes.node,
  full: PropTypes.bool,
  label: PropTypes.string
};

var UnderlineNav$1 = withSystemProps(UnderlineNav, COMMON);

function UnderlineNavLink(_ref) {
  var className = _ref.className,
      selected = _ref.selected,
      Tag = _ref.is,
      rest = objectWithoutProperties(_ref, ['className', 'selected', 'is']);

  var classes = classnames(ITEM_CLASS$1, selected && SELECTED_CLASS$1, className);

  if (typeof rest.to === 'string') {
    rest.activeClassName = SELECTED_CLASS$1;
  }

  return React.createElement(Tag, _extends({ className: classes }, rest));
}

UnderlineNavLink.defaultProps = {
  is: 'a'
};

UnderlineNavLink.propTypes = {
  is: PropTypes.oneOfType([PropTypes.string, PropTypes.func]),
  selected: PropTypes.bool
};

var UnderlineNavLink$1 = withSystemProps(UnderlineNavLink, COMMON);

export { theme, Block, Box, Avatar$1 as Avatar, Button$1 as Button, ButtonDanger, ButtonPrimary, ButtonOutline, ButtonLink, OcticonButton$1 as OcticonButton, Caret$1 as Caret, CaretBox, CircleOcticon, CircleBadge$1 as CircleBadge, Details$1 as Details, Dropdown$1 as Dropdown, DonutChart$1 as DonutChart, DonutSlice$1 as DonutSlice, FilterList$1 as FilterList, FilterListItem$1 as FilterListItem, FlexContainer, FlexItem, TextInput$1 as TextInput, Heading, Label$1 as Label, BranchName$1 as BranchName, Link$1 as Link, MergeStatus$1 as MergeStatus, Text, Tooltip$1 as Tooltip, CounterLabel$1 as CounterLabel, Flash$1 as Flash, StateLabel$1 as StateLabel, UnderlineNav$1 as UnderlineNav, UnderlineNavLink$1 as UnderlineNavLink };
